class Identity:

    def __init__(self, api, id, email, first_name, last_name, business_memberships, roles):
        self.api = api
        self.email = email,
        self.first_name = first_name,
        self.last_name = last_name,
        self.business_memberships = business_memberships
        self.roles = roles
